#ifndef BASIC_H
#define BASIC_H

#include <vector>
#include <string>
#include <list>
#include "media_file.h"
#include "algorithm.h"

using namespace std;

class MainFrame;

#define MAX_ROWS 3

class PixalcoApp : public wxApp
{
public:
    PixalcoApp();
    virtual ~PixalcoApp();
	virtual bool OnInit();
    
    void AddMediaFile(const string& displayName_, const string& fileName_);
    MediaFile * GetMediaFile(const string& displayName_);
    void RemoveMediaFile(const string& displayName_);
    void RemoveAllMediaFiles(bool updateMainFrame_=false);
    void SelectMediaFile(const string& displayName_);

    void RunMovie(const string& displayName_, const string& algorithmName_);
    void StopMovie(const string& displayName_);
    void StepMovie(const string& displayName_, const string& algorithmName_);
    void SeekMovie(const string& displayName_, long pos_);
    
    void AddAlgorithm(Algorithm *algorithm_);
    Algorithm* GetAlgorithm(const string& name_);
    void ExecuteAlgorithm(const string& name_);
    void RemoveAlgorithm(const string& name_);
    void RemoveAllAlgorithms();

    void AddPlugin(const string& fileOnly_, const string& fileAndPath_);

    // age_ == 0 is current, otherwise from buffer
    wxImage* GetImage(int row_, int age_);
    // get the image from row_ or a newly created image which will have same width/heigth as orig_
    wxImage* GetImageSameType(int row_, wxImage* orig_);
    bool IsSameImageType(wxImage* a_, wxImage* b_);
    void SetImageBufferSize(int size_);
    int GetImageBufferSize()    { return mImageBufferSize; }

    // the movie has it's own buffer with images which were not yet 
    wxImage* GetImageFromMovieBuffer(int age_);
    void SetMovieBufferSize(int size_);
    int GetMovieBuffersSize()   { return mMovieBufferSize; }
    void ClearMovieBuffer();    

protected:
    void AddImageObjToMovieBuffer(ImageObject * imgObj_);
    void AddImageObjToBuffer(ImageObject * imgObj_, int line_, int row_);
    ImageObject* GetImageObjFromBuffer(int line_, int row_, int age_);
    void SetMediaFile(MediaFile * info_);
    void ClearBuffer(int line_, int row_);

private:
    MainFrame *         mMainFrame;
    vector<MediaFile>   mMediaFiles;
    vector<Algorithm*>  mAlgorithms;
    list<ImageObject*>  *mImageObjBuffer[MAX_ROWS]; // TODO: needs to support each line and row
    list<ImageObject*>  mMovieBuffer;
    ImageObject         *mCopies[MAX_ROWS];         // needed to copy images from current scopes before execution of algorithms and put them in the image buffer afterwords

    int mImageBufferSize;
    int mMovieBufferSize;
};

DECLARE_APP(PixalcoApp);
#define PIXALCO static_cast<PixalcoApp&>(wxGetApp())

#endif

