// based on www.inb.uni-luebeck.de/~boehme/avcodec_sample.cpp
// This version was written by Michael Zeilfelder
#ifndef VIDEO_H
#define VIDEO_H

class wxImage;
class wxString;
struct AVFormatContext;
struct AVCodecContext;
struct AVCodec;
struct AVFrame;
struct AVStream;
struct AVPacket;

// You need to create a VideoManager if you want to use the Video class.
class VideoManager
{
public:
    // You can only have one instance of this class
    // (not exactly a singleton, having no instance is ok if don't want to do videos)
    static VideoManager* Create();
    static void Destroy();
    
protected:
    VideoManager();
    virtual ~VideoManager();

private:
    static VideoManager * mThis;
};

class Video
{
public:
    Video();
    ~Video();
    
    bool LoadFile(const char* filename_);
    void GetSize(int & width, int &height);
    bool GetNextFrame( wxImage * targetImage_ );
    bool Seek(long frame_);
    long GetDuration(); // in frames
    long GetPosition(); // in frames
    double GetFrameRate();  // frames per second
     
protected:
    void CloseFile();
    void DebugStreamInfo( AVStream * stream_);

    int64_t TimeUnitsToFrame(int64_t units_);
    int64_t FrameToTimeUnits(int64_t frame_);

private:
    AVFormatContext     *mFormatCtx;
    AVCodecContext      *mCodecCtx;
    AVCodec             *mCodec;
    AVFrame             *mFrame; 
    AVFrame             *mFrameRGB;
    AVPacket            *mLastPacket;
    
    int                 mVideoStream;
    long                mPosition;
    int64_t             mStartTime;
};

#endif // VIDEO_H
